import { useEffect, useState } from 'react'
import { Link } from 'react-router-dom'

import TagSearch from '@/components/TagSearch'
import SubsectionTitle from '@/components/common/SubsectionTitle'
import SidebarComponent from '@/components/layout/SidebarComponent'
import { CATEGORIES } from '@/consts'

import type { Category, Post, Tag } from '@/types'

type TagByCategory = { [key in Category]: Tag[] }

type Props = { post: Post | null }


export default ({ post }: Props) => {
  const [tags, setTags] = useState({ } as TagByCategory)

  const categoryNames: Partial<{ [key in Category]: string }> = {
      general:    '一般',
      deerjikist: 'ニジラー',
      nico:       'ニコニコタグ' }

  useEffect (() => {
    if (!(post))
      return

    const fetchTags = () => {
      const tagsTmp = { } as TagByCategory
      for (const tag of post.tags)
        {
          if (!(tag.category in tagsTmp))
            tagsTmp[tag.category] = []
          tagsTmp[tag.category].push (tag)
        }
      for (const cat of Object.keys (tagsTmp) as (keyof typeof tagsTmp)[])
        tagsTmp[cat].sort ((tagA: Tag, tagB: Tag) => tagA.name < tagB.name ? -1 : 1)
      setTags (tagsTmp)
    }

    fetchTags ()
  }, [post])

  return (
      <SidebarComponent>
        <TagSearch />
        {CATEGORIES.map ((cat: Category) => cat in tags && (
            <div className="my-3" key={cat}>
              <SubsectionTitle>{categoryNames[cat]}</SubsectionTitle>
              <ul>
                {tags[cat].map (tag => (
                    <li key={tag.id} className="mb-1">
                      <Link to={`/posts?${ (new URLSearchParams ({ tags: tag.name })).toString () }`}>
                        {tag.name}
                      </Link>
                    </li>))}
              </ul>
            </div>))}
      </SidebarComponent>)
}
